from typing import Optional, Type, Any
from pydantic.v1 import BaseModel, Field

from embedchain import App
from embedchain.models.data_type import DataType

from ..rag.rag_tool import RagTool

class FixedTXTSearchToolSchema(BaseModel):
	"""Input for TXTSearchTool."""
	search_query: str = Field(..., description="Mandatory search query you want to use to search the txt's content")

class TXTSearchToolSchema(FixedTXTSearchToolSchema):
	"""Input for TXTSearchTool."""
	txt: str = Field(..., description="Mandatory txt path you want to search")

class TXTSearchTool(RagTool):
	name: str = "Search a txt's content"
	description: str = "A tool that can be used to semantic search a query from a txt's content."
	summarize: bool = False
	args_schema: Type[BaseModel] = TXTSearchToolSchema
	txt: Optional[str] = None

	def __init__(self, txt: Optional[str] = None, **kwargs):
		super().__init__(**kwargs)
		if txt is not None:
			self.txt = txt
			self.description = f"A tool that can be used to semantic search a query the {txt} txt's content."
			self.args_schema = FixedTXTSearchToolSchema

	def _run(
		self,
		search_query: str,
		**kwargs: Any,
	) -> Any:
		txt = kwargs.get('txt', self.txt)
		self.app = App()
		self.app.add(txt, data_type=DataType.TEXT_FILE)
		return super()._run(query=search_query)


from typing import Optional, Type, Any
from pydantic.v1 import BaseModel, Field

from embedchain import App
from embedchain.models.data_type import DataType

from ..rag.rag_tool import RagTool


class FixedCSVSearchToolSchema(BaseModel):
	"""Input for CSVSearchTool."""
	search_query: str = Field(..., description="Mandatory search query you want to use to search the CSV's content")

class CSVSearchToolSchema(FixedCSVSearchToolSchema):
	"""Input for CSVSearchTool."""
	pdf: str = Field(..., description="Mandatory csv path you want to search")

class CSVSearchTool(RagTool):
	name: str = "Search a CSV's content"
	description: str = "A tool that can be used to semantic search a query from a CSV's content."
	summarize: bool = False
	args_schema: Type[BaseModel] = CSVSearchToolSchema
	csv: Optional[str] = None

	def __init__(self, csv: Optional[str] = None, **kwargs):
		super().__init__(**kwargs)
		if csv is not None:
			self.csv = csv
			self.description = f"A tool that can be used to semantic search a query the {csv} CSV's content."
			self.args_schema = FixedCSVSearchToolSchema

	def _run(
		self,
		search_query: str,
		**kwargs: Any,
	) -> Any:
		csv = kwargs.get('csv', self.csv)
		self.app = App()
		self.app.add(csv, data_type=DataType.CSV)
		return super()._run(query=search_query)


from abc import ABC, abstractmethod
from typing import Any, Callable, cast, Optional, Type

from pydantic import BaseModel, model_validator
from pydantic.v1 import BaseModel as V1BaseModel

from langchain_core.tools import StructuredTool

class BaseTool(BaseModel, ABC):
    name: str
    """The unique name of the tool that clearly communicates its purpose."""
    description: str
    """Used to tell the model how/when/why to use the tool."""
    args_schema: Optional[Type[V1BaseModel]] = None
    """The schema for the arguments that the tool accepts."""

    @model_validator(mode="after")
    def _check_args_schema(self):
        self._set_args_schema()
        return self

    def run(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        print(f"Using Tool: {self.name}")
        return self._run(*args, **kwargs)

    @abstractmethod
    def _run(
        self,
        *args: Any,
        **kwargs: Any,
    ) -> Any:
        """Here goes the actual implementation of the tool."""

    def to_langchain(self) -> StructuredTool:
        self._set_args_schema()
        return StructuredTool(
            name=self.name,
            description=self.description,
            args_schema=self.args_schema,
            func=self._run,
        )

    def _set_args_schema(self):
        if self.args_schema is None:
            class_name = f"{self.__class__.__name__}Schema"
            self.args_schema = type(
                class_name,
                (V1BaseModel,),
                {
                    "__annotations__": {
                        k: v for k, v in self._run.__annotations__.items() if k != 'return'
                    },
                },
            )


class Tool(BaseTool):
    func: Callable
    """The function that will be executed when the tool is called."""

    def _run(self, *args: Any, **kwargs: Any) -> Any:
        return self.func(*args, **kwargs)


def to_langchain(
    tools: list[BaseTool | StructuredTool],
) -> list[StructuredTool]:
    return [t.to_langchain() if isinstance(t, BaseTool) else t for t in tools]


def tool(*args):
    """
    Decorator to create a tool from a function.
    """

    def _make_with_name(tool_name: str) -> Callable:
        def _make_tool(f: Callable) -> BaseTool:
            if f.__doc__ is None:
                raise ValueError("Function must have a docstring")

            args_schema = None
            if f.__annotations__:
                class_name = "".join(tool_name.split()).title()
                args_schema = type(
                    class_name,
                    (V1BaseModel,),
                    {
                        "__annotations__": {
                            k: v for k, v in f.__annotations__.items() if k != 'return'
                        },
                    },
                )

            return Tool(
                name=tool_name,
                description=f.__doc__,
                func=f,
                args_schema=args_schema,
            )

        return _make_tool

    if len(args) == 1 and callable(args[0]):
        return _make_with_name(args[0].__name__)(args[0])
    if len(args) == 1 and isinstance(args[0], str):
        return _make_with_name(args[0])
    raise ValueError("Invalid arguments")


import requests
from bs4 import BeautifulSoup
from typing import Optional, Type, Any
from pydantic.v1 import BaseModel, Field
from ..base_tool import BaseTool

class FixedScrapeWebsiteToolSchema(BaseModel):
	"""Input for ScrapeWebsiteTool."""
	pass

class ScrapeWebsiteToolSchema(FixedScrapeWebsiteToolSchema):
	"""Input for ScrapeWebsiteTool."""
	website_url: str = Field(..., description="Mandatory website url to read the file")

class ScrapeWebsiteTool(BaseTool):
	name: str = "Read a website content"
	description: str = "A tool that can be used to read a website content."
	args_schema: Type[BaseModel] = ScrapeWebsiteToolSchema
	website_url: Optional[str] = None
	headers: Optional[dict] = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'}

	def __init__(self, website_url: Optional[str] = None, **kwargs):
		super().__init__(**kwargs)
		if website_url is not None:
			self.website_url = website_url
			self.description = f"A tool that can be used to read {website_url}'s content."
			self.args_schema = FixedScrapeWebsiteToolSchema

	def _run(
		self,
		**kwargs: Any,
	) -> Any:
		website_url = kwargs.get('website_url', self.website_url)
		page = requests.get(website_url, headers=self.headers)
		parsed = BeautifulSoup(page.content, "html.parser")
		return parsed.get_text()

